package com.example.demo.service;

import com.example.demo.model.Car;
import com.example.demo.repository.CarRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.OffsetScrollPosition;
import org.springframework.data.domain.ScrollPosition;
import org.springframework.data.support.WindowIterator;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class CarService {

    @Autowired
    private CarRepository repository;

    public List<Car> findAll() {
        return repository.findAll();
    }

    /*
    The provided method, <code>findCarsUsingOffsetFilteringAndWindowIterator</code>, is designed to retrieve a list of cars based on a specified year while implementing a form of paging or windowing mechanism. It takes a single parameter, <code>year</code>, representing the target year for car retrieval. The method employs a custom <code>WindowIterator</code> to abstract the process of fetching data in manageable chunks or windows. Inside the <code>WindowIterator.of(...)</code>, there's a lambda expression responsible for retrieving the next window of car data from the repository. This lambda invokes the <code>repository.findFirst3ByModelYear(year, (OffsetScrollPosition) scrollPosition)</code> method, which appears to fetch the first three cars for the given year, utilizing a <code>scrollPosition</code> to determine the starting point for the next window of data. The initial position is set using <code>ScrollPosition.offset()</code>. The method iterates through the windows, accumulating cars in an <code>ArrayList</code> named <code>cars</code>. Finally, it returns the <code>cars</code> list, which should contain all the cars retrieved from the database. This approach enables the retrieval of potentially large datasets in smaller, memory-efficient windows, improving performance when dealing with substantial data volumes.
     */
    public List<Car> findCarsUsingOffsetFilteringAndWindowIterator(String year) {
        final WindowIterator<Car> carWindowIterator =
                WindowIterator.of(scrollPosition -> repository.findFirst3ByModelYear(year, (OffsetScrollPosition) scrollPosition))
                        .startingAt(ScrollPosition.offset());

        final List<Car> cars = new ArrayList<>();
        carWindowIterator.forEachRemaining(cars::add);
        return cars;
    }
}
